clear all; clc; close all;

% Generate Table

warning('off', 'all');
addpath("../");
databases       = {'DSI1','DSI2','LIB1','LIB2','MAN1','MAN2','SIM001','TUT1','TUT2','TUT3','TUT4','TUT5'};
models          = {'knn_baseline','knn_dbscan', 'knn_dbscanmod','knn_dbscanmodpca'};
rep = 1;
sep = filesep();
results = "";
table_results = cell(size(databases,2)+1,20);

for db = 1:size(databases,2)
  for model = 1:size(models,2)
    
    %DB  info
    load(['databases' sep databases{db} '.mat']);
    table_results(db,1) = databases{db};
    table_results(db,2) = size(database.trainingMacs,1);
    table_results(db,3) = '-';
    
    folderClusters = ['Clusters' sep databases{db} sep 'knn_dbscan' sep] ;
    mdl = strrep(models{model}, "knn", "clusters");
    
    % Result .mat file
    [best_distance,best_rep,best_k,need_execution,Eps,MinPts,numberOfDimensions] = getOptimalParams(databases{db},3);
      
    if strcmp(models{model},'knn_baseline')
      [best_distance,best_rep,best_k,need_execution] = getOptimalParams(databases{db},2);
      results = ['Results' sep databases{db} sep models{model} sep  best_rep '_' strrep(best_distance,'distance_','') '_k' sprintf('%03d',best_k) sep 'results_rep' sprintf('%03d',rep) '.mat'] ;
      if ~exist(results)
        fprintf('File not found : %s\n', results );
      else
        load(results);
        table_results(db,4) = sprintf('%3.2f',mean(results.error((results.error(:,4)==0),1)));
        table_results(db,5) = sprintf('%3.2f',sum(results.timesample(:,3)-results.timesample(:,2)) + sum(results.timesample(:,4)-results.timesample(:,3)));
        table_results(db,6) = 1;
        table_results(db,7) = 1;
      end      
    elseif strcmp(models{model},'knn_dbscan')
      results = ['Results' sep databases{db} sep models{model} '_eps_' sprintf('%.03f',Eps) '_minpts_'  sprintf('%03d',MinPts) sep best_rep '_' strrep(best_distance,'distance_','') '_k' sprintf('%03d',best_k) sep 'results_rep' sprintf('%03d',rep) '.mat'] ;
      [best_distance,best_rep,best_k,need_execution,Eps,MinPts,numberOfDimensions] = getOptimalParams(databases{db},3);
      cluster = [folderClusters, mdl,'_',best_rep,'_eps_', sprintf('%.03f',Eps), '_minpts_', sprintf('%03d',MinPts),'.mat'];
      if ~exist(cluster)
        fprintf('File not found : %s\n', cluster );
      else
        load(results);
        load(cluster);
        
        table_results(db,8) = size(unique(nonzeros(clusters.idx)),1);
        table_results(db,9) = size(clusters.idx(clusters.idx==0),1);
        table_results(db,10) = sprintf('%3.2f',(mean(results.error((results.error(:,4)==0),1)))/str2double(table_results(db,4)));
        table_results(db,11) = sprintf('%3.4f',(sum(results.timesample(:,3)-results.timesample(:,2)) + sum(results.timesample(:,4)-results.timesample(:,3)))/str2double(table_results(db,5)));
      end  
    elseif strcmp(models{model},'knn_dbscanmod')
      results = ['Results' sep databases{db} sep models{model} '_eps_' sprintf('%.03f',Eps) '_minpts_'  sprintf('%03d',MinPts) sep best_rep '_' strrep(best_distance,'distance_','') '_k' sprintf('%03d',best_k) sep 'results_rep' sprintf('%03d',rep) '.mat'] ;
      [best_distance,best_rep,best_k,need_execution,Eps,MinPts,numberOfDimensions] = getOptimalParams(databases{db},3);
      cluster = [folderClusters, mdl,'_',best_rep,'_eps_', sprintf('%.03f',Eps), '_minpts_', sprintf('%03d',MinPts),'.mat'];
      if ~exist(cluster)
        fprintf('File not found : %s\n', cluster );
      else
        load(results);
        load(cluster);
        table_results(db,12) = size(unique(nonzeros(clusters.idx)),1);
        table_results(db,13) = size(clusters.idx(clusters.idx==0),1);
        table_results(db,14) = sprintf('%3.2f',(mean(results.error((results.error(:,4)==0),1)))/str2double(table_results(db,4)));
        table_results(db,15) = sprintf('%3.4f',(sum(results.timesample(:,3)-results.timesample(:,2)) + sum(results.timesample(:,4)-results.timesample(:,3)))/str2double(table_results(db,5)));
      end 
    elseif strcmp(models{model},'knn_dbscanmodpca')
      [best_distance,best_rep,best_k,need_execution,Eps,MinPts,numberOfDimensions] = getOptimalParams(databases{db},4);
      results = ['Results' sep databases{db} sep models{model} '_eps_' sprintf('%.05f',Eps) '_minpts_'  sprintf('%03d',MinPts) '_dim_' sprintf('%03d',numberOfDimensions) sep best_rep '_' strrep(best_distance, 'distance_','') '_k' sprintf('%03d',best_k) sep 'results_rep' sprintf('%03d',rep) '.mat'] ;
      folderClusters = ['Clusters' sep databases{db} sep 'knn_dbscanmodpca' sep] ;
      [best_distance,best_rep,best_k,need_execution,Eps,MinPts,numberOfDimensions] = getOptimalParams(databases{db},4);
      cluster = [folderClusters, mdl,'_',best_rep,'_eps_', sprintf('%.05f',Eps), '_minpts_', sprintf('%03d',MinPts),'_dim_',sprintf('%03d',numberOfDimensions),'.mat'];
      if ~exist(cluster)
        fprintf('File not found : %s\n', cluster );
      else
        load(results);
        load(cluster);
        table_results(db,16) = sprintf('%d',numberOfDimensions);
        table_results(db,17) = size(unique(nonzeros(clusters.idx)),1);
        table_results(db,18) = size(clusters.idx(clusters.idx==0),1);
        table_results(db,19) = sprintf('%3.2f',(mean(results.error((results.error(:,4)==0),1)))/str2double(table_results(db,4)));
        table_results(db,20) = sprintf('%3.4f',(sum(results.timesample(:,3)-results.timesample(:,2)) + sum(results.timesample(:,4)-results.timesample(:,3)))/str2double(table_results(db,5)));
      end 
    end
  end
end


for i = [6,7,10,11,14,15,19,20]
  val = 0;
  for j = 1:size(databases,2)
    val = val + str2double(table_results{j,i});
  end
  table_results(size(databases,2)+1,i) = val/size(databases,2);
end
table_results(size(databases,2)+1,6) = 1;
table_results(size(databases,2)+1,7) = 1;

cell2csv('ResultsReport/csv/results2.csv', table_results, false, ',', 2000, '.');